**************************************************************************;
* Project: TreeScan Censoring                                             ;
* Program: tree_extraction_SCRI.sas                                       ;
* Version:V1.4.0                                                          ;
* Date : 12/10/2017                                                       ;
* Input data: INDATA: INDATA.&demtable                                    ;
*                    INDATA.&enrtable                                     ;
*                    INDATA.&diatable                                     ;
*                    INDATA.&PROCTABLE                                    ;
*                    INDATA.&distable                                     ; 
*                    INDATA.&deathtable                                   ;
*  			  Files provided with the program:                            ;
*                    site_info                                            ;
*                    general_input                                        ;
*                    general_parameters                                   ;
*                    subgroup                                             ;
*                    child_parent(dxtree_input)                           ;
*                    dxtree_priority(optional)                            ;  
*                    id10-id9_map(optional - depending on TW)             ;
*                    concomitant_group                                    ;
*                    group_exclusion                                      ;
*                    AE_enc_setting                                       ;  
*                    wash_up_enc_setting                                  ;
*             For run with PS option some input files will be omitted     ;                                                                  ;
*                    Macros: create_episodes_spans_2.sas(by Yury Vilk)    ; 
*                            convert_child_parent.sas(by David Cole)      ;
*                            isdata.sas                                   ;
*                            ms_delpatients.sas                           ; 
*                            ms_time_signature.sas                        ;
*                            macro_create_priority                        ;
*                            macro_create_age_strata                      ;
*                            ms_time_signature                            ;
*                                                                         ;
* Output data to be sent along with *log file:                            ;
*                 MSOC.self_control_orig_dx                               ;
*                 MSOC.elig_exp_doses                                     ;
*                 MSOC.exp_wkday                                          ;
*                 MSOC.exp_age                                            ;  
*                 MSOC.dx_wkday                                           ;  
*                 msoc.orig_dx_wkday                                      ;
*				  msoc.exposure_group_process_flow                        ;
*				  msoc.exposure_group_level_outc                          ;
*                 msoc.ms_time_signature                                  ;
* For run with PS option - there are output files for each MATCHID value  ;
**************************************************************************;

%ms_starttimer(START=MAINSTART);

*** Put general parameters into macro variables to use across the program;

proc sql noprint;
  select STRIP(PUT(ps_flag,1.)), STRIP(PUT(priority_file,1.)), STRIP(PUT(id10_flag,1.)), sex, age_begin, age_finish, age_period, timestrat, 
        tree_dx_codetype, runid
                into :ps_flag,:priority_file,:id10_flag,:sex,:age_begin,:age_finish,:age_period,:timestrat,:tree_dx_codetype,:runid
    from &general_parameters.;
quit; 

*** Site specific input information;

proc sql noprint;
  select cohort_start_dt FORMAT=8. , cohort_end_dt FORMAT=8. ,aso_excl_flag into:cohort_start_dt,:cohort_end_dt,: aso_excl_flag
    from &site_info.
	  WHERE UPCASE(DPID)=UPCASE("&DPID.")
	  		AND   UPCASE(siteid)=UPCASE("&siteid.")
		;
 TITLE "Cohort Dates" ;
quit; 

 * Set defaults and do some defensive coding for general input variables;

data general_parameters;
  set &GENERAL_parameters.;
      Age_begin=COALESCE(Age_begin,0);
	  Age_finish=COALESCE(Age_finish,160);
	  AGE_PERIOD=COALESCEC(AGE_PERIOD,"YEAR");

RUN;
 
%global exit_flag;
%let exit_flag=0; 

%macro check_flags;

*** To check requester & sites inputs regarding exclusion ASO population;

*** If needed  - could be used for any exclusion list - just to be clear the name of the flag and file probably should be changed;

%PUT aso_excl_flag=&aso_excl_flag.;

 %ISDATA(dataset=&ASO_EXCL_LIST.);

 %MACRO chk_aso_excl;

 %IF  &aso_excl_flag.=1 AND  &NOBS.=0  %THEN
              %DO;
	  %PUT WARNING: The  aso_excl_flag=1 but the dataset ASO_EXCL_LIST does not exist or has no records ! ;
	  %let exit_flag=1;
			   %END;                
%MEND chk_aso_excl;

%chk_aso_excl;


%PUT id10_flag=&id10_flag.;

 %ISDATA(dataset=&id10_id9_map.);

 %MACRO chk_id10;

 %IF  &id10_flag.=1 AND  &NOBS.=0  %THEN
              %DO;
	  %PUT WARNING: The  id10_flag=1 but the dataset id10_id9_map does not exist or has no records ! ;
	   %let exit_flag=1;
			   %END;                 
%MEND chk_id10;

 %chk_id10;

%if &exit_flag = 1 %then %do;
        %put One or more required datasets  do not exist or have no observations. The program is aborted;
		%goto exit;
%end;
	

 * Process input tables;

 * Set defaults and do some defensive coding for general input variables;

DATA GENERAL_INPUT;
    SET &GENERAL_INPUT.;
	
	IF	 Medcov=" " THEN Medcov="Y";	
	 Medcov=UPCASE(Medcov) ;
	 Drugcov=UPCASE(Drugcov) ;
	  Incident_level=COALESCE(Incident_level,3);
	  Enr_gap=COALESCE(Enr_gap,45);
	  Exp_wash_up=COALESCE(Exp_wash_up,127);
	  AE_wash_up=COALESCE(AE_wash_up,183);
	  F_up_window_st=COALESCE(F_up_window_st,1);
	  F_up_window_end=COALESCE(F_up_window_end,56);
	  birth_enr_dist_allowed=COALESCE(birth_enr_dist_allowed,30);
	  enr_birth_grace_period=COALESCE(enr_birth_grace_period,45);
	  if AE_wash_up<F_up_window_end then do;
	  PUT 'WARNING: The length of AE_wash_up should be >= F_up_window_end !!!';  
	  end;
RUN;

data grp_inc_encounter_setting;
 set &AE_enc_setting.;
   Incident_enc_setting=UPCASE(Incident_enc_setting);
   PDX=upcase(PDX);
run;

data wash_up_enc_setting;
 set &wash_up_enc_setting.;
   wash_up_enc_setting=UPCASE(wash_up_enc_setting);
   PDX=upcase(PDX);
   excl_wash_up_flag=1;
run;

proc sql;
 create table uniq_exp_grp as
	  select distinct exposure_group, exposure_group_name
	    from &general_input.;
quit;


%convert_vertical_tree(  ChildParentDSName	=&dxtree_input.
						  	 , DxNodeDSName			=dplocal.nodetree
							 , LevelDlm				=%str(.) 
						     );



****** Modified to be able to run the code with or without ICD10;

	%Macro checkid10;

data id9_dxtree_input;
  set dplocal.nodetree;
  dx=upcase(compress(dx,'.'));
  orig_dx=dx;
  orig_dx_codetype=dx_codetype;
run;

%if &id10_flag=1 %then %do; 

data dxtree_input;
  set id9_dxtree_input &id10_id9_map.;
  run;
%end;

%else %do;

data dxtree_input;
  set id9_dxtree_input;
run;

%end;

%mend checkid10;

%checkid10;

proc datasets library=work nolist;
  delete id9_dxtree_input;
run;
 

*** To check if priority file exist and to run the macro to create one if it doesnt; 

%PUT priority_file=&priority_file.;

%global dxtree_priority;

 %MACRO chk_priority;

 %IF  &priority_file.=1 %THEN %DO;

%LET  dxtree_priority=infolder.dxtree_priority;

%ISDATA(dataset=&dxtree_priority.);

 %if &NOBS.=0  %THEN
              %DO;
	  %PUT WARNING: The  priority_file=1 but the dataset dxtree_priority does not exist or has no records ! ;
	   %let exit_flag=1;
			   %END;    
%END; 

%ELSE %DO;
	%create_priority;
	%LET  dxtree_priority=dplocal.dxtree_priority;
%END;

%MEND chk_priority;

%chk_priority;

%if &exit_flag = 1 %then %do;
        %put One or more required datasets  do not exist or have no observations. The program is aborted;
		%goto exit;
%end;
	

  *** Attach priority to all tree diagnosis of interest for each exposure group according to list of this group parameters;

  PROC SQL ;
    CREATE TABLE  dx_mlccs_for_inc_level_by_grp AS
	  SELECT   A.exposure_group
                     , A.Incident_level
					  ,B.node
	                  ,A.DX
                      ,A.DX_CODETYPE
					  ,A.priority                  
	      FROM 	          &dxtree_priority.          A
		  INNER JOIN  dplocal.nodetree               B
		                   ON upcase(compress(A.DX,'.'))=upcase(compress(B.DX,'.'))
						AND  A.DX_CODETYPE=B.DX_CODETYPE
						AND A.Incident_level=B.level
				ORDER BY A.exposure_group
                                   ,B.node
								   ,A.priority
								   ,B.DX
					;
	QUIT;


*****************************************************************;


PROC SORT DATA=dxtree_input
                       OUT=dx_tree_input_codes_srt NODUPKEY;
 BY orig_dx orig_dx_codetype;
RUN;


  *** Check what code categories reflect exposure codes and extract the records 
     containing these exposure codes and dxtree codes within defined cohort time window from correspondent files;


 %macro exp;
  %LOCAL exp_dx_events_tbl_name exp_px_events_tbl_name exp_rx_events_tbl_name;

data dx_exp_codes 
		px_exp_codes 
        rx_exp_codes(rename=(code=NDC))
    ;

  set &subgroup.;
	  if         code_category='DX' then output dx_exp_codes;
	  else if code_category='PX' then output px_exp_codes;
	  else if code_category='RX' then output rx_exp_codes;
run;
   
proc sql NOPRINT;
  select STRIP(PUT(count(*),12.)) as nobs into:dx_codes_cnt
    from dx_exp_codes;
 select STRIP(PUT(count(*),12.)) as nobs into:px_codes_cnt
    from px_exp_codes;
 select STRIP(PUT(count(*),12.)) as nobs into:rx_codes_cnt
    from rx_exp_codes;
 quit;

/* *** To put number of exposure codes of each category into log;*/

 %PUT dx_codes_cnt=&dx_codes_cnt.;  
 %PUT px_codes_cnt=&px_codes_cnt.; 
 %PUT rx_codes_cnt=&rx_codes_cnt.; 

/*** if dx codes are listed as exposure  - extract correspondent data from INDATA diagnosis file;*/
/*** Otherwise extract the data for tree diagnosis only;*/



%if &dx_codes_cnt ne 0 %then %do;


PROC SORT DATA=dx_exp_codes(rename=(code=orig_dx code_type=orig_dx_codetype) drop=code_category) 
						OUT=dx_exp_codes_srt ;
 BY orig_dx orig_dx_codetype;
RUN;

/*** Combine exposure and tree diags by merge to avoid duplicates during extraction process;*/
data dx_all_input_codes;
  MERGE dx_tree_input_codes_srt(IN=intree) 
                dx_exp_codes_srt(IN=inexp)
      ;
    BY orig_dx orig_dx_codetype;
	 in_tree=intree;
	 in_exp=inexp;

	 IF subgroup=. THEN subgroup=-1;	  
  run;
    %LET exp_dx_events_tbl_name=exp_dx_events;
%END;

%else %do;
data dx_all_input_codes;
  set dx_tree_input_codes_srt;
   in_tree=1;
   in_exp=0;
   subgroup=-1;	
  run; 
     %LET exp_dx_events_tbl_name=%STR();
  %end;

  proc datasets library=work nolist;
    delete dx_tree_input_codes_srt;
	run;


  PROC FREQ DATA=dx_all_input_codes;
   TABLES in_tree*in_exp*subgroup/LIST NOCUM;
    TITLE "Sources of DX codes";
   RUN; 

   
 /*** To create the list of unique diagnosis specific to the data partner from INDATA diag file ;*/

proc sql;
   create table unique_mscdm_dx_codes as
   select distinct DX, dx_codetype
     from INDATA.&diatable
;
 quit;
  		   			
	
/*** From all unique DPs diagnosis select only those presented in tree or exposure list per request;*/
	
proc sql;
  create table dx_input_codes_in_mscdm_form as
  select a.dx
            ,a.dx_codetype,b.subgroup
			,b.in_tree,b.in_exp
   from unique_mscdm_dx_codes as    a
		  , dx_all_input_codes                 as b
     where compress(upcase(a.dx),'. ')=compress(upcase(b.orig_dx),'. ') and a.dx_codetype=b.orig_dx_codetype;
quit;

proc datasets library=work nolist;
 delete unique_mscdm_dx_codes dx_all_input_codes;
run; 
	 	
/*** Extract all events with defined list of diagnosis of interest;*/
	
proc sql;
  create table all_dx_events as
  select  a.patid, a.adate,a.dx, a.dx_codetype, a.enctype, a.pdx
		    , b.subgroup
			,b.in_tree,b.in_exp
    from INDATA.&diatable                                               as a
		  , dx_input_codes_in_mscdm_form                        as b 
	  where a.dx=b.dx and a.dx_codetype=b.dx_codetype
		 AND &cohort_start_dt.<=adate<=&cohort_end_dt.
		;
quit;

proc datasets library=work nolist;
delete dx_input_codes_in_mscdm_form;
run;
 
/*** Split extracted events into separate files - tree and exposure;*/
	
	   data exp_dx_events(keep=patid adate dx dx_codetype subgroup) 
				tree_dx_events(keep= patid adate enctype dx PDX dx_codetype rename=(dx=orig_dx dx_codetype=orig_dx_codetype));
	     set all_dx_events;

		 if in_exp then output exp_dx_events;
		  if in_tree then output tree_dx_events;          
		run;
 

proc datasets library=work nolist;
delete all_dx_events;
 run;


/*** if px codes are listed as exposure  - extract data from INDATA procedure file;*/
/*** Find events of interest similar to dx exposure events above; */

%if &px_codes_cnt ne 0 %then %do;

 proc sql;
   create table unique_mscdm_px_codes as
   select distinct px
     from INDATA.&PROCTABLE
   ;
 quit;

proc sql;
  create table px_input_codes as
  select DISTINCT a.*
                            ,b.code_type  as px_codetype
                            ,b.subgroup
   from unique_mscdm_px_codes as a
		  ,px_exp_codes                as b
     where compress(upcase(a.px),'. ')=compress(upcase(b.code),'. ');
quit;

proc datasets library=work nolist;
delete unique_mscdm_px_codes;
 run;

proc sql;
  create table exp_px_events as
  select b.subgroup, a.patid, a.adate,a.px, a.px_codetype
    from INDATA.&PROCTABLE as a
          , px_input_codes              as b
	  where   a.px=b.px and a.px_codetype=b.px_codetype
		  AND &cohort_start_dt.<=adate<=&cohort_end_dt.
			;
	   quit;


proc datasets library=work nolist;
delete px_input_codes;
 run;

	     %LET exp_px_events_tbl_name=exp_px_events;
%end;
%else  %LET exp_px_events_tbl_name=%STR();



	 /*** if rx codes are listed as exposure  - extract data from INDATA dispensing file;*/
/*** Find events of interest similar to dx & px exposure events above; */

%if &rx_codes_cnt ne 0 %then %do;


proc sql;
  create table exp_rx_events(rename=(rxdate=adate)) as
  select b.subgroup, a.patid, a.rxdate, a.NDC
    from INDATA.&distable as a
            ,rx_exp_codes as b                                                          
	  where a.NDC=b.NDC 
	  AND &cohort_start_dt.<=rxdate<=&cohort_end_dt.
   ;
 quit;

   	   %LET exp_rx_events_tbl_name=exp_rx_events;
%end;
%else  %LET exp_rx_events_tbl_name=%STR();

*** Create unique combination of group & subgroups;

proc sql;
  create table grp_subgrp as
    select distinct group, subgroup 
      from &CONCOMITANT_GROUP.;
quit;

/*** Create exposure events structure file with 0 observation, but containing columns of interest;*/

DATA all_exp_events_structure;
   SET INDATA.&diatable(OBS=0 KEEP=patid adate dx dx_codetype )
	        INDATA.&PROCTABLE(OBS=0 KEEP=px px_codetype )
            INDATA.&distable(OBS=0 KEEP=NDC)
			grp_subgrp(OBS=0 KEEP=subgroup)
	;
 RUN;


 /*** Combine all exposure events;*/

DATA all_exp_events1;
  SET all_exp_events_structure
          &exp_dx_events_tbl_name.
          &exp_px_events_tbl_name.
          &exp_rx_events_tbl_name.
    ;
RUN;

 %ISDATA(dataset=indata.&deathtable.);

 %IF  &NOBS.=0  %THEN
              %DO;
	  %PUT WARNING: Death file does not exist or has no records ! ;

	 data all_exp_events;
	     set all_exp_events1;	
         deathdt=.; 
     run;
	 
proc datasets library=work nolist;
  delete all_exp_events1;
   run;
	     %END;    
 
%ELSE %DO;

*** To attach death information and delete records with exposures date after reported death date; 

proc sql;
  create table all_exp_events2 as
  select distinct a.*, b.deathdt
    from all_exp_events1 as a left join indata.&deathtable. as b
	  on a.patid=b.patid;
	   quit;

proc datasets library=work nolist;
  delete all_exp_events1;
   run;

data all_exp_events;
	     set all_exp_events2;
		   if deathdt ne . and adate>deathdt then delete;
run;


proc datasets library=work nolist;
  delete all_exp_events2;
   run;

%END;

/*** Exclude ASO patients if (properly) requested (it means if flag=1 then the file should exist);*/

%IF  &aso_excl_flag.=1 %THEN 
                                  %DO;
%ms_delpatients(datafile=all_exp_events, ptsfile=&ASO_EXCL_LIST.,    
                               outfile=all_exp_events);
							      %END;

%mend exp;


%macro get_tree_dx;

 /*** To create the list of unique diagnosis specific to the data partner from INDATA diag file ;*/

proc sql;
   create table unique_mscdm_dx_codes as
   select distinct DX, dx_codetype
     from INDATA.&diatable
;
 quit;
  		   			
	
/*** From all unique DPs diagnosis select only those presented in tree or exposure list per request;*/
	
proc sql;
  create table dx_input_codes_in_mscdm_form as
  select a.dx, a.dx_codetype
   from unique_mscdm_dx_codes as    a
		  , dx_tree_input_codes_srt                 as b
     where compress(upcase(a.dx),'. ')=compress(upcase(b.orig_dx),'. ') and a.dx_codetype=b.orig_dx_codetype;
	 quit;

	 proc datasets library=work nolist;
	   delete unique_mscdm_dx_codes;
    run; 
	 	
/*** Extract all events with defined list of diagnosis of interest;*/
	
proc sql;
  create table tree_dx_events as
  select  a.patid, a.adate,a.dx as orig_dx, a.dx_codetype as orig_dx_codetype, a.enctype, a.pdx
    from INDATA.&diatable(IDXWHERE=NO)                          as a
		  , dx_input_codes_in_mscdm_form                        as b 
	  where a.dx=b.dx and a.dx_codetype=b.dx_codetype
		 AND &cohort_start_dt.<=adate<=&cohort_end_dt.
		;
quit;

	 proc datasets library=work nolist;
	   delete dx_input_codes_in_mscdm_form;
    run; 

%mend get_tree_dx;


*** To check if this run is linked to CIDA PS result and define how to select all exposures of interest;

%global ps_master ps_match;

%Macro select_analysis;

%if &ps_flag=1 %then %do;

%get_tree_dx;

 %let ps_master=cida.&runid._mstr;
 %let ps_match=cida.&runid._matched_1_1;

%end;

%else %exp;

%mend select_analysis;

%select_analysis; 


*** Macro to select eligible exposures if this run is not linked to CIDA PS result;

%macro exp_eligibility;


      *  Create table all_exp_events_by_exp_grp which has all parameters for specified
         concomitant group attached to each exposure record based on subgroups of inclusion;
       * It will be used in several places below;

PROC SQL;
   	 CREATE TABLE all_exp_events_by_exp_grp	AS 
	    SELECT DISTINCT  B.exposure_group,
		b.group,
		b.primary,
		b.exposure_order
		
                                         ,A.PatID
		                                ,A.ADATE
										,A.deathdt
									
				FROM              all_exp_events		    A
				INNER JOIN     &concomitant_group.       		  B
	                  ON A.subgroup=B.subgroup
				  ORDER BY 	   B.exposure_group
                                         ,A.PatID
		                                , A.ADATE
				 ;
QUIT;




*** 1.Preparation for process flow counts;

proc sql;
create table flow_exp_event as
  select distinct exposure_group, count(distinct patid) as exp_pnt_cnt, count(*) as exp_event_cnt
    from all_exp_events_by_exp_grp
 group by exposure_group;
quit;
***;


*** Limit exposure records for each exposure_group to primary exposure only ;

PROC SQL;
   	 CREATE TABLE exp_pat_group_prim	AS 
	    SELECT DISTINCT  exposure_group
                                         ,PatID, adate, deathdt
				FROM  all_exp_events_by_exp_grp
				  where primary=1 ;
QUIT;

*** 2.Preparation for process flow counts; 

proc sql;
create table flow_exp_event_prim as
  select distinct exposure_group, count(distinct patid) as prim_exp_pnt_cnt, count(*) as prim_exp_event_cnt
    from exp_pat_group_prim
 group by exposure_group;
quit;

***;


PROC SQL;
 CREATE TABLE   exp_pat_group_parms AS
   SELECT distinct 
        A.PatID
	   ,A.adate
	   ,A.deathdt
       ,B.*
	   ,C.excl_dist_start
	   ,C.excl_dist_end
	    FROM 		   exp_pat_group_prim     as        A, general_input(drop=incident_level)  as   B, &group_exclusion. as c 
		      where A.exposure_group=B.exposure_group=c.exposure_group
           
		ORDER BY  B.exposure_group, A.PatID, adate
				;
QUIT;


*** Select incident exposures only for each exposure group according to specified exp_wash_up;

data exp_pat_group_parms2;
 set exp_pat_group_parms;
  by exposure_group PatID adate;
    retain check_dt;
	if first.patid then do;
                        check_dt=adate;
                        dist=.;
						end;
	else do;
         dist=adate-check_dt;
		 check_dt=adate;
		 end;
		 
		  run;

		  data exp_pat_group_parms3;
		    set exp_pat_group_parms2;
              by exposure_group PatID adate;
                 if ~first.patid and dist<=exp_wash_up then delete;
run; 

proc datasets library=work nolist;
  delete exp_pat_group_parms exp_pat_group_parms2;
run;


*** 3.Preparation for process flow counts; 

proc sql;
  create table flow_inc_exp_event as
  select distinct exposure_group, count(distinct patid) as inc_exp_pnt_cnt, count(*) as inc_exp_event_cnt
    from exp_pat_group_parms3
	 group by exposure_group;
	  quit;

***;

*** Add enrollment and demo info to check age_eligibilty and drug coverage;

 proc sql;
  create table exp_enroll_demo_d as
  select  C.*
            ,a.ENR_START
			,a.ENR_END
            ,b.birth_date 
			,b.sex
    from  		INDATA.&enrtable as a
    INNER JOIN   exp_pat_group_parms3 as C
	   ON      a.PatID=C.PatID
	  AND (A.DRUGCOV=C.DRUGCOV OR C.DRUGCOV IS NULL)
   INNER JOIN  INDATA.&demtable as b
	  ON C.patid=b.patid 
	  and b.birth_date is not Null
	  and (upcase(b.sex)=upcase("&sex.") or "&sex." = ' ')
   ;
QUIT;

*** Roll up enrollment records with defined memgap;

%create_episodes_spans_2(tbl_name_in=exp_enroll_demo_d
                         ,tbl_name_out=roll_up_enr_d(RENAME=(enr_start_span=enr_start_dt  enr_end_span=enr_end_dt)) 
                          ,by_vars=exposure_group patid
                          ,eff_dt_var_nm=ENR_start
                          ,exp_dt_var_nm=enr_end
                          ,grace_period_var_nm=enr_gap);

proc datasets library=work nolist;
  delete exp_enroll_demo_d;
   run;

*** Select only records that meet enrollment/demo criteria;


proc sort data=roll_up_enr_d;
   by exposure_group patid;
run;

data enroll_demo_d
           excl_d;
   set roll_up_enr_d;

    LENGTH excl_reason $ 32;
	 
	FORMAT age_begin_dt  age_finish_dt mmddyy10.;

	
	   orig_enr_start_dt=enr_start_dt ;
	   flag_enr_from_birth=0;

  if enr_start_dt-birth_date<-birth_enr_dist_allowed then do;
                                                excl_reason='enr_before_dob';
                                                output excl_d;
												end;
											

	 ELSE IF 	enr_start_dt-birth_date<=enr_birth_grace_period	  then 
         DO;
      enr_start_dt=birth_date;         *Reset enr_start_dt to birth_date;   *** new;
	  flag_enr_from_birth=1;				 ;
	    END;
                    
  ELSE; * do nothing;
		
            age_begin_dt=intnx("&age_period.", birth_date, &age_begin., 'SAMEDAY');
			age_finish_dt=intnx("&age_period.", birth_date, &age_finish.+1, 'SAMEDAY')-1;

			if excl_reason=" " then do;
			if age_begin_dt>enr_end_dt or age_finish_dt<enr_start_dt then do;
			                                              excl_reason='enr_age       ';
														  output excl_d;
														   end;
														   end;

 

IF excl_reason=" " then do;
	check_enr_length=enr_end_dt-enr_start_dt;		
   output enroll_demo_d;
 end;	
RUN;


proc sql;
  create table exp_pat_group_parms3_d as
  select distinct a.*, enr_start_dt, enr_end_dt, age_begin_dt, age_finish_dt, birth_date,sex
    from exp_pat_group_parms3 as a, enroll_demo_d as b
      where a.patid=b.patid and enr_start_dt+a.exp_wash_up<=a.adate<=enr_end_dt
         order by exposure_group, patid, adate;
quit;

proc datasets library=work nolist;
  delete enroll_demo_d roll_up_enr_d exp_pat_group_parms3;
   run;

*** 4.Preparation for process flow counts;
 
proc sql;
  create table flow_drug_enr_elig as
  select distinct exposure_group, count(distinct patid) as drug_enr_demo_elig_pnt_cnt, count(*) as drug_enr_demo_exp_event_elig
    from exp_pat_group_parms3_d
	 group by exposure_group;
	  quit;

***;

proc sql;
  create table exp_enroll_demo_m as
  select  C.*
            ,a.ENR_START
			,a.ENR_END
    from  		INDATA.&enrtable as a
    INNER JOIN   exp_pat_group_parms3_d(drop=enr_start_dt enr_end_dt) as C
	   ON      a.PatID=C.PatID
	  AND upcase(A.MEDCOV)=upcase(C.MEDCOV)
 INNER JOIN  INDATA.&demtable as b
	  ON C.patid=b.patid 
	  and b.birth_date is not Null
   	and (upcase(b.sex)=upcase("&sex.") or "&sex."=' ');
QUIT;


%create_episodes_spans_2(tbl_name_in=exp_enroll_demo_m
                         ,tbl_name_out=roll_up_enr(RENAME=(enr_start_span=enr_start_dt  enr_end_span=enr_end_dt)) 
                          ,by_vars=exposure_group patid
                          ,eff_dt_var_nm=ENR_start
                          ,exp_dt_var_nm=enr_end
                          ,grace_period_var_nm=enr_gap);


proc datasets library=work nolist;
  delete exp_enroll_demo_m;
  run;
*** Select only records that meet enrollment/demo criteria;


proc sort data=roll_up_enr;
   by exposure_group patid;
run;


data enroll_demo
           excl_m;
   set roll_up_enr;

    LENGTH excl_reason $ 32;
	 
	
	   orig_enr_start_dt=enr_start_dt ;
	   flag_enr_from_birth=0;

  if enr_start_dt-birth_date<-birth_enr_dist_allowed then do;
                                                excl_reason='enr_before_dob';
                                                output excl_m;
												end;
											

	 ELSE IF 	enr_start_dt-birth_date<=enr_birth_grace_period	  then 
         DO;
      enr_start_dt=birth_date;         *Reset enr_start_dt to birth_date;   *** new;
	  flag_enr_from_birth=1;				 ;
	    END;
                    
  ELSE; * do nothing;
		
       	if excl_reason=" " then do;    
			if age_begin_dt>enr_end_dt or age_finish_dt<enr_start_dt then do;
			                                              excl_reason='enr_age       ';
														  output excl_m;
														   end;
        end;
		 IF excl_reason=" " then do;

check_enr_length=enr_end_dt-enr_start_dt;		
   output enroll_demo;	
    ENd;
RUN;

data excl;
  set excl_d excl_m;
run;

proc sort data=excl nodupkey;
  by patid enr_start_dt enr_end_dt excl_reason;
run;

*** Define contributed time periods (when exposure dates meet enrollment criteria);


data enroll_demo2 ;
  set enroll_demo;
   by exposure_group patid enr_start_dt;

  FORMAT enr_contrib_st  enr_contrib_end  mmddyy10.;

  if flag_enr_from_birth=1 then enr_contrib_st=max(&cohort_start_dt., age_begin_dt);

	  else enr_contrib_st=max(&cohort_start_dt.+max(Exp_wash_up,(AE_wash_up-F_up_window_st)), enr_start_dt+max(Exp_wash_up,(AE_wash_up-F_up_window_st)), age_begin_dt);

	  enr_contrib_end=min((&cohort_end_dt.-post_exp_min_enr), (enr_end_dt-post_exp_min_enr), age_finish_dt);

format enr_contrib_st enr_contrib_end mmddyy10.;

contrib_wind_length=enr_contrib_end-enr_contrib_st;

if deathdt ne . then censor_dt=min(enr_end_dt, &cohort_end_dt., deathdt);
else censor_dt= min(enr_end_dt, &cohort_end_dt.);

run;

proc datasets library=work nolist;
 delete enroll_demo roll_up_enr;
 run;

	***  Select only exposures for patient of appropriate age with eligible enrollment, including post_exp_min_enr,  within cohort tw;
*var patid &cohort_start_dt enr_start_dt age_begin_dt enr_contrib_end f_up_window_end;
* some variables added for validation purposes only;

proc sql;
  create table exp as
             select  a.PatID,a.ADATE, b.censor_dt
                       ,b.exposure_group, b.birth_date, b.sex, b.enr_start_dt
                       ,b.enr_contrib_st, b.enr_contrib_end, b.flag_enr_from_birth
                      ,"&age_period.", b.AE_wash_up, b.f_up_window_st, b.f_up_window_end, b.exp_wash_up
					  , b.enr_end_dt, b.age_begin_dt, b.age_finish_dt, b.enr_gap
    from 			 exp_pat_group_parms3_d as a
    INNER JOIN enroll_demo2   as b
	                 ON a.patid=b.patid
					AND a.exposure_group=b.exposure_group
					AND enr_contrib_st<=a.adate<=b.enr_contrib_end

           order by b.exposure_group, a.patid, a.adate
              ;
quit;


*** Create set of flags to check criteria;

 

data exp2(rename=(adate=exp_dt)) ;
  set exp;
    by exposure_group patid adate;
	exp_wkday=weekday(adate);

      time_to_censor=censor_dt-adate;
	  age_at_exp_dt=intck("&age_period.",birth_date, adate, 'c');
	  
 NBSTRAT=0; 
  do while(scan("&TIMESTRAT.",(NBSTRAT+1),' ') ne ' ');
    NBSTRAT+1;
  end;

  if nbstrat>0 then do;
%age_strat1(start_dt=birth_date, end_dt=adate, age_strat="&timestrat.");
end;

if age_group=' ' then age_group=put(age_at_exp_dt, 3.);

* age_group=put(age_at_exp_dt, age_grp.);

	 flag_right_age_at_exp_dt=(age_begin_dt<=adate<=age_finish_dt);
	 flag_enr_at_exp_dt=(enr_start_dt<=adate<=enr_end_dt);
	 flag_enr_ramp_at_exp_dt=(flag_enr_at_exp_dt AND adate-enr_start_dt>=max(EXP_wash_up,(AE_wash_up-F_up_window_st)));
	 flag_enr_at_f_up_window_st=(enr_start_dt<=adate+f_up_window_st<=enr_end_dt);
	 flag_enr_at_f_up_window_end=(enr_start_dt<=adate+f_up_window_end<=enr_end_dt);
	 
	
run;

 proc freq data=exp2;
  tables age_group time_to_censor;
  run;

*** 5.Preparation for process flow counts;

proc sql;
  create table flow_med_enr_elig as
  select distinct exposure_group, count(distinct patid) as med_enr_demo_elig_pnt_cnt, count(*) as med_enr_demo_exp_event_elig
    from exp2
	 group by exposure_group;
quit; 

****;


				 TITLE1  "Combination of flag_enr_from_birth*flag_enr_ramp_at_exp_dt should have 1 for at least one of them";
				  TITLE2 "All other flags should be 1" ;

PROC FREQ DATA=exp2; 

  TABLES  flag_enr_from_birth*flag_enr_ramp_at_exp_dt
				 flag_right_age_at_exp_dt  flag_enr_at_exp_dt 
                  flag_enr_at_f_up_window_st  flag_enr_at_f_up_window_end
                  /MISSING LIST;
RUN;

%mend exp_eligibility;

*** Macro to apply exposure exclusion criteria;

 %MACRO chk_excl;

%IF  &NOBS. ne 0  %THEN %DO;

*  Create table exp_events_excl which has exposure_group attached to exposure records based on exclusion subgroups; 

proc sql;
create table exp_events_excl as
  select distinct  B.exposure_group
  				, excl_dist_start
				,excl_dist_end
		          ,A.PatID
		          ,A.ADATE
				 
				FROM              all_exp_events		    A
				INNER JOIN     &group_exclusion       		  B
	                  ON A.subgroup=B.subgroup
				  ORDER BY 	   B.exposure_group
                                         ,A.PatID
		                                , A.ADATE;
quit;


*** Exclude records with prior exposures based on exclusion criteria;

proc sql;
  create table exp_grp_excl as
  select distinct a.*, b.adate as excl_adate
    from exp2 as a left join exp_events_excl as b
       on a.exposure_group=b.exposure_group and a.patid=b.patid and a.exp_dt+excl_dist_start<=b.adate<=a.exp_dt+excl_dist_end;
create table exp3 as
		select *
		  from exp_grp_excl
		    where excl_adate=.;
			quit;

 %END;   
 
%else %do;
 
	 data exp3;
	  set exp2;
	  run;

%end; 

proc datasets library=work nolist;
	delete exp exp2 exp_grp_excl;
run;
  
 %MEND;



%Macro select_analysis2;

%if &ps_flag ne 1 %then %do;

%exp_eligibility;

%ISDATA(dataset=&group_exclusion);

 %chk_excl;


*** 6.Preparation for process flow counts;

proc sql;
  create table flow_elig_after_excl as
  select distinct exposure_group,  count(distinct patid) as elig_pnt_cnt_after_excl, count(*) as exp_event_elig_after_excl
    from exp3
	 group by exposure_group;
quit;
 
***;

***** Combine patient and exposure events counts through all logical steps;

proc sql;
  create table msoc.exposure_group_process_flow as
  select distinct a.exposure_group_name, a.exposure_group, b.exp_pnt_cnt, b.exp_event_cnt, c.prim_exp_pnt_cnt, c.prim_exp_event_cnt,
  d.inc_exp_pnt_cnt, d.inc_exp_event_cnt, e.drug_enr_demo_elig_pnt_cnt, e.drug_enr_demo_exp_event_elig,
  f.med_enr_demo_elig_pnt_cnt, f.med_enr_demo_exp_event_elig, g.elig_pnt_cnt_after_excl, g.exp_event_elig_after_excl               
    from general_input as a, flow_exp_event as b, flow_exp_event_prim as c, flow_inc_exp_event as d,
	    flow_drug_enr_elig as e, flow_med_enr_elig as f, flow_elig_after_excl as g
		   where a.exposure_group=b.exposure_group=c.exposure_group=d.exposure_group=e.exposure_group=f.exposure_group=g.exposure_group
             order by exposure_group;
quit;

*********************************************************************************************;

proc datasets library=work nolist;
  delete  flow_exp_event flow_exp_event_prim flow_inc_exp_event 
	    flow_drug_enr_elig flow_med_enr_elig flow_elig_after_excl;
run;
 


*** Add non_primary exposure to the selected primary ones for each exposure_group;

proc sql;
     create table exposure_dt_all as
	 select distinct a.*, b.age_at_exp_dt, b.age_group, b.sex, "&age_period.", 1 as flag, b.enr_start_dt, b.enr_end_dt
	  from all_exp_events_by_exp_grp as a, exp3 as b
	   where a.exposure_group=b.exposure_group and a.patid=b.patid and a.adate=b.exp_dt
 order by patid, adate, exposure_group ;
quit;

proc sql;
  create table combo_template as
  select distinct a.patid, a.adate, a.age_at_exp_dt, a.age_group,a.sex, "&age_period.", a.enr_start_dt, a.enr_end_dt, b.exposure_group,
                 b.exposure_order, 0 as flag
    from exposure_dt_all as a, &concomitant_group. as b
 order by patid , adate, exposure_group;
	quit;

proc sql;
     create table exposure_dt_all_2 as
	 select distinct a.*,  sum(a.flag, b.flag) as combo_flag
	  from combo_template as a left join exposure_dt_all as b
	    on a.exposure_group=b.exposure_group & a.exposure_order=b.exposure_order & a.patid=b.patid and a.adate=b.adate 
 order by patid, adate, exposure_group, exposure_order, age_at_exp_dt, age_group;
quit;

 proc transpose data=exposure_dt_all_2(drop=exposure_order) out=exposure_dt_all_3(drop= _name_) prefix=V;

by patid adate enr_start_dt enr_end_dt exposure_group age_at_exp_dt age_group sex;
   var combo_flag;
    run;

proc sql;
  select max(exposure_order) as num into:num
    from &concomitant_group.;
	quit;

%macro get_exp_comb;
 data exposure_dt_all_4;
  length exp_comb $&num.;
  set exposure_dt_all_3;
  where v1=1;

  exp_comb=' ';
    %do i=1 %to &num.;
     exp_comb=compress(exp_comb||v&i);
     %end;


		exp_wkday=weekday(adate);
	 
	 run;
%mend;

%get_exp_comb;

proc datasets library=work nolist;
  delete exposure_dt_all exposure_dt_all_2 exposure_dt_all_3;
   run;

*** Define number of eligible exposure doses for each group of interest;

 

*** This file should be sent to MSOC;
proc sql;
 create table MSOC.elig_exp_doses as
  select distinct a.exposure_group, b.exposure_group_name, exp_comb, age_group, count(*) as elig_exp_dose_cnt
    from exposure_dt_all_4 as a, uniq_exp_grp as b
	 where a.exposure_group=b.exposure_group
      group by a.exposure_group, exp_comb, age_group;
quit;


*** Define the frequency of the day of the week and patient age at the day of exposure;
*** Result files should be sent to MSOC;

proc sort data=exposure_dt_all_4;
 by exposure_group  exp_comb;
  run;

proc freq data=exposure_dt_all_4 noprint;
  by exposure_group  exp_comb;
  tables exp_wkday/missing out=msoc.exp_wkday;
 tables age_at_exp_dt/missing out=msoc.exp_age;
 run;


 *** Define the time period of diagnosis consideration;

 data enroll_demo4;
   set exp3;
     
     AE_win_end_dt=exp_dt+f_up_window_end;
;


   * Calculate  outcome of interest start date. ; 
   outcome_win_start_dt=exp_dt+f_up_window_st;

 age_at_outc_win_start_days=outcome_win_start_dt-birth_date; **** new;

 
if flag_enr_from_birth=1 & age_at_outc_win_start_days<AE_wash_up then AE_win_start_dt=birth_date;	   **** new;
 ELSE AE_win_start_dt=outcome_win_start_dt-AE_wash_up;                                              **** new;


   * The outcome end date is the same as AE_win_end_dt;
   
  format birth_date AE_win_start_dt AE_win_end_dt outcome_win_start_dt mmddyy10.;
  run;
 
  proc datasets library=work nolist;
    delete exp3;
  run;

%end;


%ELSE %DO;

data ps_master;
  set &ps_master.( keep=group patid sex indexdt enr_start enr_end age agegroup 
                  rename=(group=exposure_group_name indexdt=adate age=age_at_exp_dt agegroup=age_group enr_start=enr_start_dt enr_end=enr_end_dt));  
  time_to_censor=enr_end_dt-adate;
  exp_wkday=weekday(adate);
run;

proc sql;
  create table ps_match as
  select distinct a.*, b.exposure_group
    from &ps_match.(rename=(studyclass=exposure_group_name indexdt=adate age_cat=age_group)) as a, general_input as b
	 where a.exposure_group_name=b.exposure_group_name;
quit;

data ps_matched(keep=patid exposure_group exposure_group_name adate predefinedpsmatchid1_1 hdpspredefinedmatchid1_1 hdpsonlymatchid1_1);
  length predefinedpsmatchid1_1 hdpspredefinedmatchid1_1 hdpsonlymatchid1_1 $50;
  set ps_match;
    predefinedpsmatchid1_1=COALESCEC(predefinedpsmatchid1_1,' ');
	 hdpspredefinedmatchid1_1=COALESCEC(hdpspredefinedmatchid1_1,' ');
	  hdpsonlymatchid1_1=COALESCEC(hdpsonlymatchid1_1,' ');
run;

proc datasets library=work nolist;
  delete ps_match;
   run;

proc sql;
  create table ps_master_general as
  select distinct a.*, b.*
    from ps_master as a, general_input(rename=(exposure_group_name=exp_name) drop=incident_level) as b
	  where upcase(left(a.exposure_group_name))=upcase(left(b.exp_name));
  create table ps_master_match as
  select distinct a.*, predefinedpsmatchid1_1, hdpspredefinedmatchid1_1, hdpsonlymatchid1_1
    from ps_master_general(drop=exp_name) as a left join ps_matched as b
	on a.exposure_group=b.exposure_group & a.patid=b.patid;
quit;


proc datasets library=work nolist;
  delete ps_master_general;
   run;

proc sql;
 create table MSOC.elig_exp_doses as
  select distinct exposure_group, exposure_group_name, age_group, count(*) as elig_exp_dose_cnt
    from ps_master_match
      group by exposure_group, age_group;
quit;


*** Define the frequency of the day of the week and patient age at the day of exposure;
*** Result files should be sent to MSOC;

proc sort data=ps_master_match;
 by exposure_group;
  run;

proc freq data=ps_master_match noprint;
  by exposure_group;
  tables exp_wkday/missing out=msoc.exp_wkday;
 tables age_at_exp_dt/missing out=msoc.exp_age;
 run;


 *** Define the time period of diagnosis consideration;
 data enroll_demo4;
   set ps_master_match(rename=(adate=exp_dt));
     
     AE_win_end_dt=min(enr_end_dt, (exp_dt+f_up_window_end));


   * Calculate  outcome of interest start date. ; 
   outcome_win_start_dt=exp_dt+f_up_window_st;

	AE_win_start_dt=outcome_win_start_dt-AE_wash_up;                                             


   * The outcome end date is the same as AE_win_end_dt;
   
  format  AE_win_start_dt AE_win_end_dt outcome_win_start_dt mmddyy10.;
  run;


%END;

%mend select_analysis2;

%select_analysis2; 


				   *Process outcomes (dx tree events); 


proc sql ;
  create table dx_tree_events_for_exp_pat as
  select DISTINCT A.PATID, A.ADATE, A.orig_DX, A.orig_DX_CODETYPE, A.ENCTYPE,a.pdx, b.dx, b.dx_codetype
    from tree_dx_events	 a, dxtree_input as b
	  where a.patid in
	   (select patid
	    from enroll_demo4) and compress(upcase(a.orig_dx),'.')=compress(upcase(b.orig_dx),'.')
              and a.orig_dx_codetype=b.orig_dx_codetype
 order by dx, dx_codetype;
quit;
	
proc datasets library=work nolist;
	delete dxtree_input tree_dx_events;
 run;

PROC SQL ;
  CREATE TABLE dx_tree_exp_group_events as
  SELECT 		 b.exposure_group
			,A.PATID
                        ,A.ADATE
						,A.orig_dx
						,A.orig_dx_codetype
						,A.PDX
                        ,b.node
                        ,b.incident_level   
			,b.priority
			,b.DX
                        ,b.DX_CODETYPE 
                        ,A.ENCTYPE

    FROM   dx_tree_events_for_exp_pat as a
          INNER JOIN    dx_mlccs_for_inc_level_by_grp as b
	                ON compress(upcase(a.dx),'. ')=compress(upcase(b.dx),'. ')
                   AND a.dx_codetype=b.dx_codetype  
      ORDER BY b.exposure_group, a.patid, a.adate,b.node,b.priority, b.dx, b.dx_codetype, 
               b.incident_level;
quit;
 

proc datasets library=work nolist;
	delete dx_tree_events_for_exp_pat dx_mlccs_for_inc_level_by_grp;
 run;

proc sql;
  create table diag_demo as
  select distinct b.exposure_group, a.patid, a.adate, a.enctype, a.pdx, a.node ,A.orig_dx
						,A.orig_dx_codetype, a.dx, a.dx_codetype,a.incident_level   
						,a.priority, b.sex, b.enr_start_dt, b.exp_dt, b.AE_win_start_dt, b.f_up_window_st, b.f_up_window_end,  b.AE_wash_up
						,b.outcome_win_start_dt, b.time_to_censor
    from dx_tree_exp_group_events as a
           , enroll_demo4     as b 
	  where a.exposure_group=b.exposure_group  
             & a.patid=b.patid 
            & b.AE_win_start_dt<=a.adate<=AE_win_end_dt
                 ;  
quit;
 

proc datasets library=work nolist;
	delete dx_tree_exp_group_events;
 run;

*** Select potential incident diagnosis;

proc sql;
create table diag_demo_pot_inc as
	  select a.*
	    from diag_demo(where=( adate>=outcome_win_start_dt)) as a
               ,grp_inc_encounter_setting as c
		  where a.exposure_group=c.exposure_group 
                  & ((a.enctype=c.incident_enc_setting and (a.PDX=c.PDX or c.PDX IS NULL)) OR c.incident_enc_setting IS NULL) 
		 ORDER BY exposure_group, patid,exp_dt, adate, node, priority
      ;
quit;


*** Select the highest priority diagnosis;

		  data diag_demo3;
		   set diag_demo_pot_inc;
		     by exposure_group patid exp_dt adate node priority;
			  if first.node;
			  run;


proc datasets library=work nolist;
  delete diag_demo_pot_inc;
   run;


*** To check all possible prior dx to potential incident diagnosis of the same incident level; 

proc sql;
  create table diag_demo4_A as
  select distinct a.*
                       , b.adate as prior_dt format=mmddyy10.
                       , (a.adate-b.adate) as days_from_prev_AE
                       , b.enctype as prior_enctype
					   , b.PDX as prior_pdx
    from     diag_demo3 as a 
   left join diag_demo   as b
	  on a.patid=b.patid 
    and a.node=b.node 
    and a.adate>b.adate
 order by exposure_group, patid, exp_dt, node, adate, days_from_prev_AE;
quit;

proc datasets library=work nolist;
  delete diag_demo diag_demo3;
   run;

*** To check if prior dx met enctype and PDX wash up criteria;

proc sql;
  create table diag_demo4_B as
   select distinct a.*, b.wash_up_enc_setting as excl_enctype, b.pdx as excl_pdx, max(excl_wash_up_flag, 0) as excl_wash_up_flag
     from diag_demo4_A as a left join wash_up_enc_setting as b
	   on a.exposure_group=b.exposure_group & 
          ((prior_enctype=wash_up_enc_setting & (prior_pdx=b.PDX or b.PDX is NULL)) or b.wash_up_enc_setting is NULL)
         order by exposure_group, patid, exp_dt, node, adate, days_from_prev_AE ;
quit;


proc datasets library=work nolist;
  delete diag_demo4_A;
   run;

*** To create exclusion flag for prior diagnosis based on distance from potential incident dx and exclusion encounter settings;
 proc freq data=diag_demo4_B;
   tables excl_wash_up_flag;
    run;

data diag_demo4 ;
 set diag_demo4_B;
   if days_from_prev_AE=. then wash_up_dist_flag=0;
   else wash_up_dist_flag=(days_from_prev_AE<=AE_wash_up);
   excl_due_to_prev_AE_flag=wash_up_dist_flag*excl_wash_up_flag;
   run;

proc datasets library=work nolist;
  delete diag_demo4_B;
   run;

   proc sort data=diag_demo4;
     by exposure_group patid node adate excl_due_to_prev_AE_flag;
   run;

*** Select incident diagnosis if there were no prior ones within requested AE wash_up_period and accoding to wash_up_enc_setting
 on the same MLCCS level (incident_level) defined in the general input file;

   data diag_demo5 excluded_due_to_prev_AE;
     set diag_demo4;
	 by exposure_group patid node adate excl_due_to_prev_AE_flag;
	 	days_from_exp_to_outcome=(adate-exp_dt);
       dx_wkday=weekday(adate);
	     if time_to_censor>f_up_window_end then time_to_censor=9999;
	 if last.adate then do;	
       if excl_due_to_prev_AE_flag=0 then output diag_demo5;
	   else output excluded_due_to_prev_AE;
	   end;
run;

proc datasets library=work nolist;
  delete diag_demo4;
   run;

%macro select_analysis3;

%if &ps_flag ne 1 %then %do;

***** Create table with patient and event counts for exposure/AE pairs by exposure_group and specified incident_level;
  

 proc sql;
   create table msoc.exposure_group_level_outc as
   select distinct  b.exposure_group_name, a.exposure_group,a.incident_level, count(distinct patid) as exp_diag_pnt_cnt, count(*) as exp_diag_event_cnt
     from diag_demo5 as a, general_input as b
	  where a.exposure_group=b.exposure_group and a.incident_level=b.incident_level
	   group by a.exposure_group, a.incident_level
 order by exposure_group, incident_level;
	   quit;


*** Attach exposure concomitant information for each primary exposure date;


proc sql;
  create table diag_demo6 as
  select distinct a.*, b.exp_comb, age_at_exp_dt, age_group
    from diag_demo5 as a, exposure_dt_all_4 as b
	  where a.patid=b.patid & a.exp_dt=b.adate & a.exposure_group=b.exposure_group
        order by exposure_group, incident_level, exp_comb;
quit;
 

*** Define the weekdays distribution of incident diagnosis;
*** The result should be sent to MSOC;

proc freq data=diag_demo6 noprint ;
  by exposure_group incident_level exp_comb;
  tables dx*dx_wkday/missing out=msoc.dx_wkday;
    tables orig_dx*dx_wkday/missing out=msoc.orig_dx_wkday;
run;



*** Create crosswalk  - the file with list of variables needed for future data freeze
    for potential statistical(TreeScan) signal investigation;
*** Kept at DP;

 proc sql;
    create table dplocal.crswlk_for_freeze as
       select distinct exposure_group, incident_level, exp_comb, patid,age_at_exp_dt, node, orig_dx, orig_dx_codetype, dx, dx_codetype, 
              adate as dx_adate ,exp_dt as exp_adate, time_to_censor
         from diag_demo6;
 quit;


*** Create analytic result file for later use as an input for TreeScan software;

 proc sql;
	    create table MSOC.self_control_orig_dx as
   select distinct "&DPID" as DPID
              ,"&siteid" as siteid
              ,a.exposure_group
			  ,b.exposure_group_name
			  ,a.incident_level
              ,a.exp_comb
			  ,a.time_to_censor
			  ,a.age_group
			  ,a.orig_dx
              ,a.orig_dx_codetype
              ,a.dx
              ,a.dx_codetype
              ,a.days_from_exp_to_outcome
               ,count(*) as exp_dx_dist_cnt
   from diag_demo6 as a, uniq_exp_grp as b
    where a.exposure_group=b.exposure_group
    group by a.exposure_group, a.incident_level,exp_comb, time_to_censor, age_group,orig_dx, orig_dx_codetype, days_from_exp_to_outcome
       order by a.exposure_group, a.incident_level, exp_comb, time_to_censor, age_group, orig_dx, orig_dx_codetype, days_from_exp_to_outcome;	 
 quit;

 proc datasets library=work nolist;
 delete diag_demo6;
  run;


 %end;

 %ELSE %DO;

 proc sql;
 create table ps_matched_outc as
 select distinct a.exposure_group, a.exposure_group_name,a.patid, predefinedPSmatchID1_1, HDPSPredefinedmatchID1_1, HDPSOnlymatchID1_1, b.incident_level,
                  a.age_at_exp_dt, a.age_group, a.enr_end_dt, a.adate as index_date, a.time_to_censor
 from ps_master_match as a, general_input as b
  where a.exposure_group=b.exposure_group;
 
   create table ps_matched_outc_2 as
   select distinct a.*, b.node, b.adate, b.dx, b.dx_codetype, b.orig_dx, b.orig_dx_codetype, dx_wkday,b.days_from_exp_to_outcome as time_to_event           
     from ps_matched_outc as a left join diag_demo5 as b
	   on a.exposure_group=b.exposure_group and a.incident_level=b.incident_level & a.patid=b.patid;
	  
 create table varlist as
  select *
    from dictionary.columns
	  where upcase(libname)='WORK' and upcase(memname)='PS_MATCHED' ;
quit;

	   proc datasets library=work nolist;
	     delete ps_matched_outc;
       run;
 
data matchvarlist;
 set varlist(keep=name);
   where upcase(name) contains 'MATCHID';
   run;

proc sql;
     select count(distinct name) into:matchvar_num
	  from matchvarlist ;
 quit;

%let matchvar_num=&matchvar_num;

%macro cida_matchid;

%do i=1 %to &matchvar_num.;

data one;
length match $6;
  set matchvarlist;
  if &i=_N_;
  match=substr(upcase(name),1,6);
  run;

  proc sql;
    select name, MATCH into: matchvar,:MATCH
	  from one;
	  quit;

	 
		  proc sql;
            create table temp_&match. as
			select distinct &matchvar., min(time_to_censor) as time_to_censor
			   from ps_matched_outc_2
				where &matchvar. ne ' ' and dx ne ' '
				  group by &matchvar.;
		   select count(*) into: obs_nbr
		    from temp_&match ;
			quit;


   %if &obs_nbr. ne 0 %then %do;

          proc sql;
			create table dplocal.treeextraction_&match. as
			select distinct a.*, b.time_to_censor
			   from ps_matched_outc_2(drop=time_to_censor rename=(time_to_event=days_from_exp_to_outcome)) as a, temp_&match. as b
				where a.&matchvar.=b.&matchvar. and days_from_exp_to_outcome<=time_to_censor and dx ne ' '
 order by exposure_group, incident_level;
				 quit;
				
*** The result should be sent to MSOC;
	
proc sql;
 create table MSOC.elig_exp_doses_&match. as
  select distinct exposure_group, exposure_group_name, age_group, count(*) as elig_exp_dose_cnt
    from dplocal.treeextraction_&match.
      group by exposure_group, age_group;
quit;			 

proc sql;
   create table msoc.group_level_outc_&match. as
   select distinct  exposure_group_name, exposure_group,incident_level, count(distinct patid) as exp_diag_pnt_cnt, 
          count(*) as exp_diag_event_cnt
              from dplocal.treeextraction_&match.
	   group by exposure_group, incident_level
 order by exposure_group, incident_level;
quit;

proc freq data=dplocal.treeextraction_&match. noprint ;
  by exposure_group incident_level ;
  tables dx*dx_wkday/missing out=msoc.dx_wkday_&match.;
    tables orig_dx*dx_wkday/missing out=msoc.orig_dx_wkday_&match.;
run;

*** Create crosswalk  - the file with list of variables needed for future data freeze
    for potential statistical(TreeScan) signal investigation;
*** Kept at DP;

 proc sql;
    create table dplocal.crswlk_for_freeze_&match. as
       select distinct exposure_group, incident_level,patid, &matchvar., age_at_exp_dt, age_group,
              node, orig_dx, orig_dx_codetype, dx, dx_codetype, adate as dx_adate ,index_date as exp_adate,time_to_censor, enr_end_dt
         from dplocal.treeextraction_&match.;
 quit;

   
*** Create analytic result file for later use as an input for TreeScan software;

 proc sql;
	    create table MSOC.self_control_orig_dx_&match. as
   select distinct "&DPID" as DPID
              ,"&siteid" as siteid
              ,a.exposure_group
			  ,a.exposure_group_name
			  ,a.incident_level
			  ,a.time_to_censor
			  ,a.age_group
			  ,a.orig_dx
              ,a.orig_dx_codetype
              ,a.dx
              ,a.dx_codetype
              ,a.days_from_exp_to_outcome
               ,count(*) as exp_dx_dist_cnt
   from dplocal.treeextraction_&match. as a
    group by a.exposure_group, a.incident_level, age_group, time_to_censor,  orig_dx, orig_dx_codetype, days_from_exp_to_outcome
       order by a.exposure_group, a.incident_level, age_group, time_to_censor, orig_dx, orig_dx_codetype, days_from_exp_to_outcome;  
 quit;
 %end;
 %end;

%mend cida_matchid;

%cida_matchid;


%END;
%mend select_analysis3;

%select_analysis3;

%exit: %mend check_flags;
%check_flags;

%ms_stoptimer(START=&MAINSTART.);
%create_signature;


